package cn.com.taiji.tile.service.impl;
import cn.com.taiji.cql.model.GeoDataSource;
import cn.com.taiji.cql.model.GeoRequestParam;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.tile.util.TileUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class TileServiceImpl implements ITileService {

    @Autowired
    private IECqlService ecqlService;


    @Override
    public byte[] tileData(GeoRequestParam requestParam,GeoDataSource geoDs) {

        //2根据cql规则获取图层对应的空间地理数据


        StringBuffer sbBbox = new StringBuffer();
        sbBbox.append("(bbox (");
        sbBbox.append(geoDs.getBboxColumn()+",");
        sbBbox.append(requestParam.getBbox()+"))");


//        String filterCql = requestParam.getCqlFilter()+" and "+sbBbox.toString();
        String filterCql = requestParam.getCqlFilter();

        Map<String, Object> cqlDsParams = new HashMap<String, Object>();
        cqlDsParams.put(JDBCDataStoreFactory.DATABASE.key, geoDs.getDataBase());
        cqlDsParams.put(JDBCDataStoreFactory.DBTYPE.key, geoDs.getDbType());

        cqlDsParams.put(JDBCDataStoreFactory.HOST.key, geoDs.getHost());
        cqlDsParams.put(JDBCDataStoreFactory.PORT.key, geoDs.getPort());
        cqlDsParams.put(JDBCDataStoreFactory.USER.key, geoDs.getUserName());
        cqlDsParams.put(JDBCDataStoreFactory.PASSWD.key, geoDs.getUserPassword());
        cqlDsParams.put("tableName", geoDs.getTableName());

        Map<String, Object> sortParams = new HashMap<String, Object>();
        sortParams.put("sort","asc");

        List<Map<String, Object>> cqlDataList = ecqlService.cqlQuery(cqlDsParams,filterCql,sortParams);

        if(cqlDataList.size() >0){
            //3根据空间地理数据生成图层二进制数据
            System.out.println("返回的数据:"+cqlDataList.toString());
            return TileUtils.getTile2lwh(cqlDataList, requestParam.getBbox(),geoDs);

        }else{

            log.info("根据cql规则:"+sbBbox.toString()+"和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
        }

        return new byte[0];
    }


}
